-------------------------------------------------------------------------------
-- Project  ELE8304 : Circuits intégrés à très grande échelle
-------------------------------------------------------------------------------
-- File     fetch_tb.vhd
-- Authors  Titouan Luard <luardtitouan@gmail.com> 
--          Yann Roberge <yann.roberge@polymtl.ca> 
-- Lab      GRM - Polytechnique Montreal
-- Date     2021-11-19
-------------------------------------------------------------------------------
-- Brief    test bench of the fetche
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

library work;
use work.riscv_pkg.all;

entity fetch_tb is
end fetch_tb;

architecture tb of fetch_tb is

    constant XLEN   : positive := 32;

    signal target    : std_logic_vector(XLEN-1 downto 0);
    signal clk       : std_logic;
    signal stall     : std_logic;
    signal transfert : std_logic;
    signal rstn      : std_logic;
    signal flush     : std_logic;
    signal mem     : std_logic_vector(XLEN-1 downto 0);
    
    signal pc        : std_logic_vector(9 downto 0);
    signal fetch     : std_logic_vector(XLEN-1 downto 0);    
    signal expected_result_pc : std_logic_vector(9 downto 0);
    signal expected_result_fetch : std_logic_vector(XLEN-1 downto 0);
    constant PERIOD   : time := 10 ns;

begin

  -- DUT
  dut: entity work.fetch
    generic map (
      RESET_VECTOR => (others => '0')
    )
    port map (
      i_target    => target,
      i_clk   => clk,
      i_stall => stall,
      i_transfert  => transfert,
      i_mem => mem,
      i_rstn  => rstn,
      o_pc => pc,
      i_flush => flush,
      o_fetch => fetch
      
    );

  drive_clk : process
  begin
    clk <= '0';
    wait for PERIOD/2;
    clk <= '1';
    wait for PERIOD/2;
  end process drive_clk;

  -- Main TB process
  p_main : process
  begin
    -- Vecteurs de tests

    report "BEGIN SIMULATION";

    report "initialisation";
    
    rstn <= '0' ;
    target <= (1 downto 0 => '1', others => '0');
    stall <= '0';
    transfert <= '0';
    
    
    
    wait for PERIOD;
    
    expected_result_pc<=(others => '0');
    expected_result_fetch<=(others => '0');    
    
    assert pc = expected_result_pc
      report "pc error" severity error;
      
    assert fetch = expected_result_fetch
      report "fetch error" severity error;
    rstn <= '1';
    flush <= '0';
    
    wait for PERIOD;

    report "first lap";
    

    stall <= '0';
    transfert <= '0';
    target <= (1 downto 0 => '1', others => '0');

    mem <= (0 downto 0 => '1', others => '0');    
    
    
    
    wait for PERIOD; 

    expected_result_pc<=(others => '0');
    
    expected_result_fetch<=(others => '0'); 
       
    assert pc = expected_result_pc
      report "pc error" severity error;
      
    assert fetch = expected_result_fetch
      report "fetch error" severity error;
      
    wait for PERIOD;
    
    report "second lap (expected +4)";
    
    rstn <= '1';
    stall <= '0';
    transfert <= '0';
    flush <= '0';   
    mem <= (0 downto 0 => '1', others => '0');
    target <=(1 downto 0 => '1', others => '0');
    wait for PERIOD; 

    expected_result_pc <= ("0000000100");
    expected_result_fetch <= (0 downto 0 => '1', others => '0'); 
       
    assert pc = expected_result_pc
      report "pc error" severity error;
      
    assert fetch = expected_result_fetch
      report "fetch error" severity error;
      
    wait for PERIOD;


    report "target";
    
    rstn <= '1';
    target <=(1 downto 0 => '1', others => '0');
    stall <='0';
    transfert <= '1';
    flush <= '0';
    mem <= (2 downto 0 => '1', others => '0');
   

  
 
    wait for PERIOD; 

    expected_result_pc <= ("0000000011");
    expected_result_fetch <= (2 downto 0 => '1', others => '0'); 
       
    assert pc = expected_result_pc
      report "pc error" severity error;
      
    assert fetch = expected_result_fetch
      report "fetch error" severity error;
      
    wait for PERIOD;
    
    report "stall";
    rstn <= '1';
    target <=(1 downto 0 => '1', others => '0');
    stall <='1';
    transfert <= '0';
    flush <= '0';
    mem <= (3 downto 0 => '1', others => '0');
   



    wait for PERIOD; 

    expected_result_pc <= ("0000000111");
    expected_result_fetch <= (2 downto 0 => '1', others => '0'); 
       
    assert pc = expected_result_pc
      report "pc error" severity error;
      
    assert fetch = expected_result_fetch
      report "fetch stall error" severity error;
      
    wait for PERIOD;
    
        wait for PERIOD;
    
    report "flush";
    rstn <= '1';
    target <=(1 downto 0 => '1', others => '0');
    stall <='0';
    transfert <= '0';
    flush <= '1';
    mem <= (3 downto 0 => '1', others => '0');
   





    wait for PERIOD; 

    expected_result_pc <= ("0000001011");
    expected_result_fetch <= (others => '0'); 
       
    assert pc = expected_result_pc
      report "pc error" severity error;
      
    assert fetch = expected_result_fetch
      report "fetch stall error" severity error;
      
    wait for PERIOD;
    
        
    report "reset";
    
    rstn <= '1' ;
    
    wait for PERIOD;
    
    expected_result_pc<=(others => '0');
    expected_result_fetch<=(others => '0');    
    
    assert pc = expected_result_pc
      report "pc error" severity error;
      
    assert fetch = expected_result_fetch
      report "fetch error" severity error;
      

    report "SIMULATION DONE";
    wait;
  end process p_main;

end architecture tb;

